﻿/* CorWX - The Core War simulator
 * Copyright (c) 2009 Petr Koupy <petr.koupy@gmail.com>
 * 
 * File: WindowMain.xaml.cs
 * 
 * Description:
 * Internal logic of window containing Setup option dialog.
 * 
 * Revision history:
 * Rev  Date        Who             Description
 * 1.0  01/05/09    Petr Koupy      Initial draft
 * 
 * GNU General Public License Usage
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace CorWX
{
    /* Setup option is showed as a dialog, thus all initialization and 
     * verification of values should be done in main window. Internal 
     * logic takes care off basic functionality of window controls.
     */
    public partial class WindowSetup : Window
    {
        public WindowSetup()
        {
            InitializeComponent();
            numericUpDownCoreSize.ValueChanged += new EventHandler(numericUpDownCoreSize_ValueChanged);
        }

        int result;
        static public int Ok = 1;
        static public int Cancel = 0;

        public int DoModal()
        {
            result = Cancel;
            ShowDialog(); 
            return result;
        }

        private void buttonOK_Click(object sender, RoutedEventArgs e)
        {
            result = Ok;
            Close();
        }

        private void buttonCancel_Click(object sender, RoutedEventArgs e)
        {
            result = Cancel;
            Close();
        }

        private void checkBoxRead_CheckedChanged(object sender, RoutedEventArgs e)
        {
            // (Un)Lock read distance to mirror core size.
            if (checkBoxRead.IsChecked == false)
                numericUpDownReadDistance.Enabled = true;
            else
                numericUpDownReadDistance.Enabled = false;
        }

        private void checkBoxWrite_CheckedChanged(object sender, RoutedEventArgs e)
        {
            // (Un)Lock write distance to mirror core size.
            if (checkBoxWrite.IsChecked == false)
                numericUpDownWriteDistance.Enabled = true;
            else
                numericUpDownWriteDistance.Enabled = false;
        }

        private void numericUpDownCoreSize_ValueChanged(object sender, EventArgs e)
        {
            /* When read and/or write distance is locked, values are same as
             * core size. When coresize is changed, read/write distance is also
             * changed.
             */
            if (checkBoxRead.IsChecked == true)
                numericUpDownReadDistance.Value = numericUpDownCoreSize.Value;
            if (checkBoxWrite.IsChecked == true)
                numericUpDownWriteDistance.Value = numericUpDownCoreSize.Value;
        }

        private void checkBoxSeparation_CheckedChanged(object sender, RoutedEventArgs e)
        {
            if (checkBoxSeparation.IsChecked == false)
            {
                numericUpDownSeparation.Enabled = true; // SEPARATION is active
                numericUpDownMinDistance.Enabled = false;
            }
            else
            {
                numericUpDownSeparation.Enabled = false; // MINDISTANCE is active
                numericUpDownMinDistance.Enabled = true;
            }
        }

        private void comboBoxInitialInstruction_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (comboBoxInitialInstruction.SelectedIndex == 0)  // CUSTOM initial instruction
            {
                comboBoxOpcode.IsEnabled = true;
                comboBoxModifier.IsEnabled = true;
                comboBoxAMode.IsEnabled = true;
                numericUpDownANumber.Enabled = true;
                comboBoxBMode.IsEnabled = true;
                numericUpDownBNumber.Enabled = true;

            }
            else                                                // NONE or RANDOM initial instruction
            {
                comboBoxOpcode.IsEnabled = false;
                comboBoxModifier.IsEnabled = false;
                comboBoxAMode.IsEnabled = false;
                numericUpDownANumber.Enabled = false;
                comboBoxBMode.IsEnabled = false;
                numericUpDownBNumber.Enabled = false;
            }
        }

        // King Of The Hill standard variable set. Can be loaded on demand.
        private void buttonKOTH_Click(object sender, RoutedEventArgs e)
        {
            numericUpDownCoreSize.Value = 8000;
            numericUpDownMaxCycles.Value = 80000;
            numericUpDownMaxLength.Value = 100;
            numericUpDownMaxProcesses.Value = 8000;
            numericUpDownMinDistance.Value = 100;
            numericUpDownReadDistance.Value = 8000;
            checkBoxRead.IsChecked = true;
            numericUpDownWriteDistance.Value = 8000;
            checkBoxWrite.IsChecked = true;
            numericUpDownSeparation.Value = 0;
            checkBoxSeparation.IsChecked = true;
            numericUpDownWarriors.Value = 2;

            comboBoxInitialInstruction.SelectedIndex = 0;
            comboBoxOpcode.SelectedItem = "DAT";
            comboBoxModifier.SelectedItem = "F";
            comboBoxAMode.SelectedItem = "$";
            numericUpDownANumber.Value = 0;
            comboBoxBMode.SelectedItem = "$";
            numericUpDownBNumber.Value = 0;
        }

        // Old ICWS86 standard variable set. Can be loaded on demand.
        private void buttonICWS86_Click(object sender, RoutedEventArgs e)
        {
            numericUpDownCoreSize.Value = 8192;
            numericUpDownMaxCycles.Value = 100000;
            numericUpDownMaxLength.Value = 300;
            numericUpDownMaxProcesses.Value = 64;
            numericUpDownMinDistance.Value = 300;
            numericUpDownReadDistance.Value = 8192;
            checkBoxRead.IsChecked = true;
            numericUpDownWriteDistance.Value = 8192;
            checkBoxWrite.IsChecked = true;
            numericUpDownSeparation.Value = 0;
            checkBoxSeparation.IsChecked = true;
            numericUpDownWarriors.Value = 2;

            comboBoxInitialInstruction.SelectedIndex = 0;
            comboBoxOpcode.SelectedItem = "DAT";
            comboBoxModifier.SelectedItem = "F";
            comboBoxAMode.SelectedItem = "#";
            numericUpDownANumber.Value = 0;
            comboBoxBMode.SelectedItem = "#";
            numericUpDownBNumber.Value = 0;
        }

        // Custom variable set loaded by default during Engine initialization.
        private void buttonDefault_Click(object sender, RoutedEventArgs e)
        {
            numericUpDownCoreSize.Value = 8000;
            numericUpDownMaxCycles.Value = 8000;
            numericUpDownMaxLength.Value = 200;
            numericUpDownMaxProcesses.Value = 400;
            numericUpDownMinDistance.Value = 200;
            numericUpDownReadDistance.Value = 8000;
            checkBoxRead.IsChecked = true;
            numericUpDownWriteDistance.Value = 8000;
            checkBoxWrite.IsChecked = true;
            numericUpDownSeparation.Value = 0;
            checkBoxSeparation.IsChecked = true;
            numericUpDownWarriors.Value = 12;

            comboBoxInitialInstruction.SelectedIndex = 0;
            comboBoxOpcode.SelectedItem = "DAT";
            comboBoxModifier.SelectedItem = "F";
            comboBoxAMode.SelectedItem = "$";
            numericUpDownANumber.Value = 0;
            comboBoxBMode.SelectedItem = "$";
            numericUpDownBNumber.Value = 0;
        }
    }
}
